{ ------------------------------------------------------------- }
{                                                               }
{   Name:           GraphCls.pas                                }
{   Version:        V 0.2                                       }
{   Author:         Erick I. Jimenez Alvarado                   }
{   To contact:     erickj@usa.net                              }
{                   http://members.xoom.com/erickj              }
{                                                               }
{   Description:    Graphics useful Classes                     }
{                                                               }
{   File Status:    COMPONETWARE ( like a FREEWARE but if you   }
{                   make a Componet or modify this please       }
{                   send or e-mail me this).                    }
{                                                               }
{                                                               }
{   Last Upadated:  01/16/99                                    }
{                                                               }
{   History:                                                    }
{                                                               }
{    01/08/99   V0.1 -  First Version                           }
{    01/16/99   V0.2 -  Add DEBUGCODE                           }
{                                                               }
{   Gratefulness: * Brad Stowers for all componets, great work  }
{                   thanks for all.                             }
{                   e-mail: bstowers@pobox.com                  }
{                   http://www.pobox.com/~bstowers/delphi/      }
{                                                               }
{   Finally:        Sorry my bad, bad english and thanks for    }
{                   use my component.                           }
{                                                               }
{ ------------------------------------------------------------- }

{.$DEFINE PRG_UNIT}

{$IFDEF PRG_UNIT}               { ---------------------- }

{$DEFINE PRG_INTERFACE}
{$DEFINE PRG_IMPLEMENTATION}

{$I XTools.Inc } { Directives for all controls }

unit GraphCls;

interface

uses

 {$IFNDEF WIN32}
    WinTypes, WinProcs,             { Default Units }
 {$ELSE}
    Windows,                        { Default Unit  }
 {$ENDIF}

    GraphExt;                       { My Units      }

{$ENDIF} { // PRG_UNIT }


{$IFDEF PRG_INTERFACE}          { ---------------------- }

type


{
// --------------------------------------------------------
//  Class to handle list of colors
// --------------------------------------------------------
}
  TColorList = Class(TObject)

  private

    FItems : Integer;
    FList  : PColorArray;

    procedure   InitArray(I: Integer);
    procedure   SetColor(Index: Integer; Value: TRGBMap);
    function    GetColor(Index: Integer): TRGBMap;

  public

    constructor Create(I:Integer); virtual;
    destructor  Destroy; override;

    procedure   Assign(Source: TColorList);
    procedure   Clear;
    function    Resize(I: Integer): Integer;

    property    Items : Integer     read FItems;     { Read only }
    property    List  : PColorArray read FList;
    property    Color[Index: Integer]: TRGBMap read GetColor write SetColor; default;

  end;

{$ENDIF} { // PRG_INTERFACE }


{$IFDEF PRG_UNIT}               { ---------------------- }

{
// --------------------------------------------------------
//
// --------------------------------------------------------
}
implementation

Uses

    SysUtils;                       { Default Units }

{$ENDIF} { // PRG_UNIT }

{$IFDEF PRG_IMPLEMENTATION}     { ---------------------- }

{
// --------------------------------------------------------
//  Create items of colors
// --------------------------------------------------------
}
constructor TColorList.Create(I: Integer);
begin
//    By Compiler
//    FItems := 0;
//    FList  := nil;

    inherited Create;

    InitArray(I);
end;

{
// --------------------------------------------------------
//  Destructor of class
// --------------------------------------------------------
}
destructor TColorList.Destroy;
begin

  {$IFDEF DEBUGCODE}

    if (FList = nil) then
        raise Exception.Create('TColorList:Destroy : Pointer Invalid');

  {$ENDIF} { // DEBUGCODE }

    Clear;

    inherited Destroy;
end;


{
// --------------------------------------------------------
//  Create items of colors
// --------------------------------------------------------
}
procedure TColorList.InitArray(I: Integer);
begin

    if (I <> 0) then
        GetMem(FList, I * SizeOf(TRGBMap));

    if (FList = nil) then
    Begin
        FItems := 0;
      {$IFDEF DEBUGCODE}
        raise Exception.Create('TColorList::InitArray: Memory Allocation Error');
      {$ENDIF} { // DEBUGCODE }
        Exit;
    End;

    FItems := I;

end;


{
// --------------------------------------------------------
//  Destroy items of colors
// --------------------------------------------------------
}
procedure TColorList.Clear;
begin

    if (FList <> nil) then
    begin
        FreeMem(FList, FItems * SizeOf(TRGBMap));
        FList   := nil;
    end;
    FItems := 0;                            { Clear all             }

end;

{
// --------------------------------------------------------
//  Resize Array of Items
// --------------------------------------------------------
}
function TColorList.Resize(I: Integer) : Integer;
begin
    Clear;                                  { Destroy Array         }
    InitArray(I);
    Result := FItems;
end;


{
// --------------------------------------------------------
//  Copy items of colors
// --------------------------------------------------------
}
procedure TColorList.Assign(Source:TColorList);
Var
    I : Integer;
begin
    I := Source.FItems;

    Clear;                                  { Destroy Array         }
    InitArray(I);                           { Create Array          }

    { Copy Array of Colors }
    if (FItems <> I) then
    begin
        Clear;

      {$IFDEF DEBUGCODE}
        raise Exception.Create('TColorList::Assign: Memory Error');
      {$ENDIF} { // DEBUGCODE }

        Exit
    end;

    CopyMemory(Pointer(FList),Pointer(Source.FList),I * SizeOf(TRGBMap));

end;


{
// --------------------------------------------------------
// Set One Color in array
// --------------------------------------------------------
}
procedure TColorList.SetColor(Index: Integer; Value: TRGBMap);

begin

    { If index is greate than Items then exit }
    if (Index > (FItems - 1)) then
    Begin
      {$IFDEF DEBUGCODE}
        raise Exception.Create('TColorList::SetColor: Index Error');
      {$ENDIF} { // DEBUGCODE }
        Exit;
    End;

    FList[Index] := Value;                      { Put New Color         }
end;

{
// --------------------------------------------------------
// Get one Color from Array Color
// --------------------------------------------------------
}
function TColorList.GetColor(Index: Integer): TRGBMap;
begin

    { If index is greate than Items then exit }
    if (Index > (FItems - 1)) then
    Begin
      {$IFDEF DEBUGCODE}
        raise Exception.Create('TColorList::GetColor: Index Error');
      {$ENDIF} { // DEBUGCODE }
        Exit;
    End;

    Result := FList[Index];                     { Return Color          }
end;

{$ENDIF} { // PRG_IMPLEMENTATION }

{$IFDEF PRG_UNIT}               { ---------------------- }
end.
{$ENDIF} { // PRG_UNIT }
