{ ------------------------------------------------------------- }
{                                                               }
{   Name:           XCtrlCls.pas                                }
{   Version:        V 0.3                                       }
{   Author:         Erick I. Jimenez Alvarado                   }
{   To contact:     erickj@usa.net                              }
{                   http://members.xoom.com/erickj              }
{                                                               }
{   Description:    Based Class to Graphics Components          }
{                                                               }
{   File Status:    COMPONETWARE ( like a FREEWARE but if you   }
{                   make a Componet or modify this please       }
{                   send or e-mail me this).                    }
{                                                               }
{   Last Upadated:  01/25/99                                    }
{                                                               }
{   History:                                                    }
{                                                               }
{    01/17/99   V0.1 -  First Version.                          }
{    01/20/99   V0.2 -  Enhanced Panit, Intercept WMPaint.      }
{    01/25/99   V0.3 -  Correct some bugs in with Palette.      }
{                                                               }
{   Finally:        Sorry my bad, bad english and thanks for    }
{                   use my component.                           }
{                                                               }
{ ------------------------------------------------------------- }

{.$DEFINE PRG_UNIT}

{$IFDEF PRG_UNIT}               { ---------------------- }

{$DEFINE PRG_INTERFACE}
{$DEFINE PRG_IMPLEMENTATION}


{$I XTools.Inc } { Directives for all controls }

Unit XCtrlCls;

interface

uses

 {$IFNDEF WIN32}
    WinTypes, WinProcs,                     { Default Units }
 {$ELSE}
    Windows,                                { Default Unit  }
 {$ENDIF}

    Classes, Controls, Graphics, Messages,  { Default Units }

    XObjects, XCompCls;

{$ENDIF} { // PRG_UNIT }


{$IFDEF PRG_INTERFACE}          { ---------------------- }


const

  CM_BackChange        = CM_BASE + 501; { When change the background of control }
  CM_TransparentChange = CM_BASE + 502; { To use when transparent control is change }


type

  TBoundsEventState  = (besSize, besPos);
  TBoundsEventStates = set of TBoundsEventState;

  TOnBoundsEvent = procedure(Sender:TObject; State: TBoundsEventStates) of object;


{
// --------------------------------------------------------
//  TEnhGraphicControl Class
// --------------------------------------------------------
}
  TEnhGraphicControl = class(TGraphicControl)
  private

    FMouseInControl : Boolean;

    FOnMouseEnter   : TNotifyEvent;
    FOnMouseLeave   : TNotifyEvent;
    FOnBoundsEvent  : TOnBoundsEvent;

    procedure   SetTransparent(Value:Boolean);
    procedure   CMTransparent(var Message: TMessage); message CM_TRANSPARENTCHANGE;

    procedure   CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure   CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure   CMBackChange(var Message: TMessage); message CM_BACKCHANGE;

    procedure   WMPaint(var Message: TWMPaint); message WM_PAINT;

  protected

    FTransparent  :  Boolean;

    {*** Misc. functions ***}

    function    VerifyInside(X , Y : Integer): Boolean; virtual;
    procedure   GetChange(Sender:TObject); virtual;
    function    GetScreenRect(R: TRect): TRect;


    {*** New Procedures ***}

    function    PutPalette(DC : HDC) : HPalette; virtual;

    procedure   BoundsChange(State : TBoundsEventStates); virtual;
    procedure   MouseEnter; virtual;
    procedure   MouseLeave; virtual;

    function    ValidRepaint: Boolean; virtual;

    {*** Override functions ***}

    procedure   SetBounds(aLeft, ATop, AWidth, AHeight: Integer); override;
    procedure   Paint;  override;
    procedure   PaintDC(DC : HDC);  virtual;

    {*** New Properties ***}

    property    OnBoundsEvent : TOnBoundsEvent read FOnBoundsEvent write FOnBoundsEvent;
    property    OnMouseEnter  : TNotifyEvent   read FOnMouseEnter  write FOnMouseEnter;
    property    OnMouseLeave  : TNotifyEvent   read FOnMouseLeave  write FOnMouseLeave;

    property    IsTransparent : Boolean        read FTransparent   write SetTransparent
                                default True;

  public

    constructor Create(Owner:TComponent); override;
    //destructor  Destroy; override;

    {*** Override functions ***}
    //procedure Invalidate; override;
    procedure   StdInvalidate; virtual;

    function    IsEmpty: Boolean; virtual;

    property    MouseInControl: Boolean read FMouseInControl;

  end;


{
// --------------------------------------------------------
//  TEnhTextureControl Class
// --------------------------------------------------------
}
  TEnhTextureControl = class(TEnhGraphicControl)
  private

    FCtrlFill     : TFillBackground;
    FBackChangeLink : TEnhChangeLink;

    procedure   SetBackground(Value : TFillBackground);
    function    ChkBackCtrl : Boolean;


  protected

    function    GetPalette : HPalette; override;
    function    PutPalette(DC : HDC) : HPalette; override;

    procedure   PaintDC(DC : HDC);  override;

  public

    constructor Create(Owner:TComponent); override;
    destructor  Destroy; override;

    function    IsEmpty: Boolean; override;

    procedure   StdInvalidate; override;

    property    CtrlFill : TFillBackground read FCtrlFill write SetBackground
                            stored ChkBackCtrl;

  end;

{$ENDIF} { // PRG_INTERFACE }

{$IFDEF PRG_UNIT}               { ---------------------- }

{
// --------------------------------------------------------
//
// --------------------------------------------------------
}
implementation
uses
     GraphExt, SysUtlEx;

{$ENDIF} { // PRG_UNIT }

{$IFDEF PRG_IMPLEMENTATION}     { ---------------------- }

{
// --------------------------------------------------------
//  Default Constructor
// --------------------------------------------------------
}
constructor TEnhGraphicControl.Create(Owner:TComponent);
begin
    inherited Create(Owner);

    { Set Control transparent by default }
    FTransparent := True;
    ControlStyle := ControlStyle - [csOpaque] + [csReplicatable];

//    Canvas.Handle := 0;             { // Relly need this ? }

    Height := 60;
    Width  := 60;
end;

{
// --------------------------------------------------------
//  Default Destructor
// --------------------------------------------------------
}
//destructor TEnhGraphicControl.Destroy;
//begin
//    inherited Destroy;
//end;

{
// --------------------------------------------------------
//  Misc. funtion to verify if point is inside in control
// --------------------------------------------------------
}
function TEnhGraphicControl.VerifyInside(X , Y : Integer): Boolean;
begin
    Result := True;
end;

{
// --------------------------------------------------------
//  Set Transparent mode and correct Style control
// --------------------------------------------------------
}
procedure TEnhGraphicControl.SetTransparent(Value : Boolean);
begin
    if (FTransparent <> Value) then
    begin
        { Correct control style }
        if (Value) then
            ControlStyle := ControlStyle - [csOpaque]
        else
            ControlStyle := ControlStyle + [csOpaque];

        FTransparent := Value;

        { Perform Correct transparent change }
        Perform(CM_TRANSPARENTCHANGE,0,0);
    end;
end;

{
// --------------------------------------------------------
//  It's call when transparent mode change
// --------------------------------------------------------
}
procedure TEnhGraphicControl.CMTransparent(var Message: TMessage);
begin
    StdInvalidate;
    //Invalidate;
end;

{
// --------------------------------------------------------
//  This procedure is assigned when component changed
// --------------------------------------------------------
}
procedure TEnhGraphicControl.GetChange(Sender:TObject);
begin
    if (csLoading in ComponentState) then
//    (csDesigning in ComponentState) then
        Exit;

    StdInvalidate;
    //Invalidate;
end;

{
// --------------------------------------------------------
//  It correct Repaint Control
// --------------------------------------------------------
}
procedure TEnhGraphicControl.StdInvalidate;
//procedure TEnhGraphicControl.Invalidate;
begin
//    if FTransparent or (csDesigning in ComponentState) then
        //inherited Invalidate    { old Invalidate }
//        Invalidate    { old Invalidate }
//    else
        RePaint;
end;

{
// --------------------------------------------------------
//  Check if is valid repaint control
// --------------------------------------------------------
}
function TEnhGraphicControl.ValidRepaint: Boolean;
begin
    Result := False;
end;

{
// --------------------------------------------------------
// Return if Control is empty to draw
// --------------------------------------------------------
}
function TEnhGraphicControl.IsEmpty: Boolean;
begin
    Result := True;
end;


{
// --------------------------------------------------------
//  Default PutPalette (Dummy)
// --------------------------------------------------------
}
function  TEnhGraphicControl.PutPalette(DC : HDC) : HPalette;
begin
    Result := 0;
end;

{
// --------------------------------------------------------
//  Default Paint (Dummy paint)
// --------------------------------------------------------
}
procedure TEnhGraphicControl.Paint;
begin
//     PaintDC(Canvas.Handle);
End;

{
// --------------------------------------------------------
//  Default PaintDC (Dummy paint)
// --------------------------------------------------------
}
procedure TEnhGraphicControl.PaintDC(DC : HDC);
begin
    if not (FTransparent) then
        FillRectEx(HDC(Pointer(DC)),Rect(0,0,Width,Height), ColorToRGB( Color));
//            //GetBkColor(TmpDC));

end;


{
// --------------------------------------------------------
//  Process Paint Message
// --------------------------------------------------------
}
procedure TEnhGraphicControl.WMPaint(var Message: TWMPaint);
var
    OldBmp: HBitmap;
    OldDC,
    TmpDC : HDC;

    OldPalette : HPalette;

begin

    if (csLoading in ComponentState) and not ValidRepaint then
        // or (csReading in ComponentState) then
        Exit;

    { Select Region default to draw, first move origin, then Intersect region }
//    SetViewportOrgEx({$IFDEF DFS_CPPB} HDC(FHandle) {$ELSE} FHandle {$ENDIF},
//                     aRect.Left,aRect.Top,nil);

//    IntersectClipRect({$IFDEF DFS_CPPB} HDC(FHandle) {$ELSE} FHandle {$ENDIF},
//               0, 0, FWidth, FHeight);

//    SetViewportOrgEx({$IFDEF DFS_CPPB} HDC(FHandle) {$ELSE} FHandle {$ENDIF},
//                     aRect.Left,aRect.Top,nil);
//    MoveWindowOrg({$IFDEF DFS_CPPB} HDC(FHandle) {$ELSE} FHandle {$ENDIF},
//                    Rect.Left,Rect.Top);
//    GetWindowOrgEx(FHandle, P);
//    SetWindowOrgEx(FHandle, P.X - aRect.Left, P.Y - aRect.Top, nil);


    { Check Mesage DC }
    if (Message.DC = 0) then
        Exit;

//    Canvas.Lock;                    { // Relly need this ? }

  try

//    Canvas.Handle := Message.DC;    { // Relly need this ? }



    // for future use
    // if (Parent is TEnhCustomControl) not create DC and BitMap
    // Beta Version in proyect

    OldDC  := Message.DC;

    TmpDC  := CreateCompatibleDC(HDC(Pointer(OldDC)));
    OldBmp := SelectObject(HDC(Pointer(TmpDC)),
              CreateCompatibleBitmap( HDC(Pointer(OldDC)), Width,Height));

   try

    { It's good trick to catch the transparent paint }

    if (FTransparent) then
    Begin
//        SelectObject(TmpDC,Canvas.Brush.Handle);
        SetBkColor(TmpDC,ColorToRGB(Color));
        SetBkMode(TmpDC, TRANSPARENT);
        BitBlt(HDC(Pointer(TmpDC)),0,0,Width,Height,
               HDC(Pointer(OldDC)),0,0,SRCCOPY);
    end
    else
    Begin
        SetBkMode(HDC(Pointer(TmpDC)), OPAQUE);
        SetBKColor(HDC(Pointer(TmpDC)), ColorToRGB(Color));
    End;


    { If is Designing draw a rectangle dummy }
    if (csDesigning in ComponentState) and isEmpty then
    Begin
        DrawBorder(HDC(Pointer(TmpDC)),Rect(0,0,Width,Height),RGBBlackColor,1);

        OldPalette := 0;
    end
    else
    begin
        OldPalette := PutPalette(TmpDC);
        PaintDC(tmpDC);
    end;

    { Put correct palette }
    PutPalette(OldDC);


    BitBlt(HDC(Pointer(OldDC)),0,0,Width,Height,
           HDC(Pointer(TmpDC)),0,0,SRCCOPY);

    if (OldPalette <> 0) then
        SelectPalette( HDC(Pointer(OldDC)), OldPalette, False);


   finally

    DeleteObject(SelectObject(HDC(Pointer(TmpDC)),OldBmp));
    DeleteDC(HDC(Pointer(TmpDC)));

   end;

  finally

//    Canvas.Handle := 0;             { // Relly need this ? }
//    Canvas.Unlock;                  { // Relly need this ? }

  end;
end;



{
// --------------------------------------------------------
//  Return Real Rect Position
// --------------------------------------------------------
}
function TEnhGraphicControl.GetScreenRect(R: TRect):TRect;
begin
    With Result do
    Begin

        TopLeft     := ClientOrigin;
        BottomRight := TopLeft;

        Inc(Left  ,R.Left);
        Inc(Top   ,R.Top);
        Inc(Right ,R.Right);
        Inc(Bottom,R.Bottom);
    End;
end;

{
// --------------------------------------------------------
//  Check if OnBoundsEvent is Assigned and Execute
// --------------------------------------------------------
}
procedure TEnhGraphicControl.BoundsChange(State: TBoundsEventStates);
begin
    if Assigned(FOnBoundsEvent) then
        FOnBoundsEvent(Self, State);
end;

{
// --------------------------------------------------------
//  Procedure to handle new Bounds and send correct mesages
// --------------------------------------------------------
}
procedure TEnhGraphicControl.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
    TmpEv : TBoundsEventStates;
    TmpR  : TRect;
begin

    TmpR := Rect(Left, Top, Width, Height);

    { Put corect bounds }
    inherited SetBounds(aLeft, aTop, aWidth, aHeight);

    { Check what event is }
    with TmpR do
    begin

        TmpEv := [];

        if (self.Left <> Left) or (Self.Top <> Top) then
            Include(TmpEv,besPos);

        if (Width <> Right) or ( Height <> Bottom) then
            Include(TmpEv,besSize);
    end;

    { Handle Event }
    if (TmpEv <> []) and not (csLoading in ComponentState) then
        BoundsChange(TmpEv);
end;

{
// --------------------------------------------------------
//  Procedure when background component change
// --------------------------------------------------------
}
procedure TEnhGraphicControl.CMBackChange(var Message: TMessage);
begin
    if (csLoading in ComponentState) then Exit;
//    if (ValidRepaint) then
        GetChange(nil);
end;


{
// --------------------------------------------------------
//  Procedure when Mouse Enter
// --------------------------------------------------------
}
procedure TEnhGraphicControl.CMMouseEnter(var Message: TMessage);
begin
    inherited;

    if ((not FMouseInControl) and Enabled and IsForegroundTask) then
//    if ((not FMouseInControl) and Enabled) then //  and IsForegroundTask) then
    begin
        FMouseInControl := True;
        if (ValidRepaint) then
            //Invalidate;                  { Repaint control       }
            Repaint;                    { Repaint control       }
        MouseEnter;                     { Execute Event handler }
    end;
end;

{
// --------------------------------------------------------
//  Procedure When Mouse Leave
// --------------------------------------------------------
}
procedure TEnhGraphicControl.CMMouseLeave(var Message: TMessage);
begin
    inherited;

//    if (FMouseInControl and Enabled) then //  and IsForegroundTask) then
    if (FMouseInControl and Enabled and IsForegroundTask) then
    Begin
        FMouseInControl := False;
        if (ValidRepaint) then
            Invalidate;                 { Repaint control       }
        MouseLeave;                     { Execute Event handler }
    End;

end;

{
// --------------------------------------------------------
//  Execute Handler on Mouse Enter
// --------------------------------------------------------
}
procedure TEnhGraphicControl.MouseEnter;
begin
    { Check if event mouse handle   }
    if Assigned(FOnMouseEnter) then
        FOnMouseEnter(self);
end;

{
// --------------------------------------------------------
//  Execute Handler on Mouse Leave
// --------------------------------------------------------
}
procedure TEnhGraphicControl.MouseLeave;
begin
    { Check if event mouse handle   }
    if Assigned(FOnMouseLeave) then
        FOnMouseLeave(self);
end;

(*
procedure TEnhGraphicControl.UpdateTracking;
var
    P: TPoint;
    OldValue: Boolean;
    TmpRect : TRect;
begin

    OldValue := FMouseInControl;

    GetCursorPos(P);

    TmpRect := GetScreenRect(ClientRect);

    P.X := P.X - TmpRect.Left;
    P.Y := P.Y - TmpRect.Top;

    { Check if mouse is real inside in control }
    FMouseInControl := Enabled and VerifyInside(P.X,P.Y);
            // (FindDragTarget(P, True) = Self) and IsForegroundTask;

    if (FMouseInControl <> OldValue) then
        if FMouseInControl then
            MouseEnter   { Repaint      }
        else
            MouseLeave;  { Invalidate   }
    end;
end;

*)

{
// --------------------------------------------------------
// --------------------------------------------------------
//  EnhTextureControl Class
// --------------------------------------------------------
// --------------------------------------------------------
}

{
// --------------------------------------------------------
//  Default Constructor
// --------------------------------------------------------
}
constructor TEnhTextureControl.Create(Owner:TComponent);
begin
    inherited Create(Owner);

    FBackChangeLink := TEnhChangeLink.Create;
    FBackChangeLink.OnChange:=GetChange;
end;

{
// --------------------------------------------------------
//  Default Destructor
// --------------------------------------------------------
}
destructor TEnhTextureControl.Destroy;
begin
    FBackChangeLink.Free;
    inherited Destroy;
end;


{
// --------------------------------------------------------
//  Set FillBackground
// --------------------------------------------------------
}
procedure TEnhTextureControl.SetBackground(Value: TFillBackground);
begin

    if (FCtrlFill <> nil) then
        FCtrlFill.UnRegisterChanges(FBackChangeLink);

    //FCtrlFill.Assign(Value);
    FCtrlFill := Value;

    if (FCtrlFill <> nil) then
    begin
        FCtrlFill.RegisterChanges(FBackChangeLink);
        FCtrlFill.FreeNotification(Self);
    end;

    // Perform(CMBackChange) ??
    GetChange(nil);
end;

{
// --------------------------------------------------------
//  Default Palette
// --------------------------------------------------------
}
function  TEnhTextureControl.GetPalette: HPALETTE;
begin
    if (FCtrlFill <> nil) then
        Result := FCtrlFill.Background.Palette
    else
        Result := inherited GetPalette;
end;

{
// --------------------------------------------------------
//  Default Self PutPalette
// --------------------------------------------------------
}
function  TEnhTextureControl.PutPalette(DC : HDC) : HPalette;
begin
    if (FCtrlFill <> nil) then
        Result := FCtrlFill.Background.PutPalette(DC)
    else
        Result := 0;
end;


{
// --------------------------------------------------------
//  Default PaintDC
// --------------------------------------------------------
}
procedure TEnhTextureControl.PaintDC(DC : HDC);
begin

    if (FCtrlFill <> nil) then
        FCtrlFill.Background.PaintDC(DC,Rect(0,0,Width,Height))
    else
        inherited PaintDC(DC);
End;


{
// --------------------------------------------------------
//  This procedure is assigned when component changed
// --------------------------------------------------------
}
procedure TEnhTextureControl.StdInvalidate;
Var
    Trans : boolean;

begin

    Trans := True;

    if (FCtrlFill <> nil) then
        Trans := FCtrlFill.Background.IsTransparent;

    isTransparent := Trans;

    inherited;

end;

{
// --------------------------------------------------------
// Return if Control is empty to draw
// --------------------------------------------------------
}
function  TEnhTextureControl.IsEmpty: Boolean;
begin
    Result := True;

    if (FCtrlFill <> nil) then
        Result := FCtrlFill.Background.IsEmpty;
end;

function  TEnhTextureControl.ChkBackCtrl : Boolean;
begin
    Result := (FCtrlFill <> nil); // and (FCtrlFill.Owner <> ManagerContainer);
end;

{$ENDIF} { // PRG_IMPLEMENTATION }

{$IFDEF PRG_UNIT}               { ---------------------- }
end.
{$ENDIF} { // PRG_UNIT }
